@@ -20,20 +20,9 @@ close(VERSION);
 @ARGV = ("../downloads/adblockplus-$version.xpi");
 do './create_xpi.pl';
 
-opendir(LOCALES, "chrome/locale");
-my @locales = grep {!/[^\w\-]/ && !-e("chrome/locale/$_/.incomplete")} readdir(LOCALES);
-closedir(LOCALES);
-
-# Create new single-locale builds
-for my $locale (@locales)
-{
-  @ARGV = ("../downloads/adblockplus-$version-$locale.xpi", $locale);
-  do './create_xpi.pl';
-}
-
 chdir('..');
 system("hg add downloads/adblockplus-$version.xpi");
-system(qq(hg commit -m "Releasing Adblock Plus $version" downloads src));
+system(qq(hg commit -m "Releasing Adblock Plus $version" downloads adblockplus));
 
 my $branch = $version;
 $branch =~ s/\./_/g;
